# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

# $Id: as_maintenance.py,v 1.24 2005/01/19 13:49:13 mcourtoi Exp $
# $Name: v1_2_1 $

# ----- differ messages translation
def _(mesg): return mesg

import os.path
import shutil
import string
import re
import glob
import time
import tarfile
from optparse import SUPPRESS_HELP

from as_config import ASTER_CONFIG
from as_build  import ASTER_BUILD
"""
Tools to maintain a development version of Code_Aster, and useful for
the developper.
Methods are called by an ASTER_RUN object.
"""

#-------------------------------------------------------------------------------
def SetParser(run):
   """Configure the command-line parser, add options name to store to the list,
   set actions informations.
   run : ASTER_RUN object which manages the execution
   """
   acts_descr={
      'getop'        : {
         'method' : GetOP,
         'syntax' : '[options] commande[.capy]',
         'help'   : 'return the main subroutine of a Code_Aster command'
      },
      'show'         : {
         'method' : Show,
         'syntax' : '[options] obj1 [obj2...]',
         'help'   : 'show a source file : fortran, C, python, capy, cata, histor or test'
      },
      'get'          : {
         'method' : Get,
         'syntax' : '[options] obj1 [obj2...]',
         'help'   : 'copy a source file in current directory'
      },
      'update'       : {
         'method' : Update,
         'syntax' : '[options] fich1.tar.gz [fich2.tar.gz...]',
         'help'   : 'perform one or several updates of a development version'
      },
      'make'         : {
         'method' : MakeAster,
         'syntax' : '[--version_dev=VERS]',
         'help'   : 'build a Code_Aster version (executable, libraries, catalogs)'
      },
      'auto_update'  : {
         'method' : AutoUpdate,
         'syntax' : '[options]',
         'help'   : 'download available updates from a server and apply them to the current development version'
      },
      'getversion'   : {
         'method' : GetVersion,
         'syntax' : '[options]',
         'help'   : 'return current release number of the development version'
      }
   }
   opts_descr={
      'local' : {
         'args'   : ('-l','--local'),
         'kwargs' : {
            'action'  : 'store_true',
            'default' : run['default_local'],
            'dest'    : 'local',
            'help'    : 'files will not been searched on a server but '+ \
                        'on the local machine'
         }
      },
      'nolocal' : {
         'args'   : ('--nolocal',),
         'kwargs' : {
            'action'  : 'store_true',
            'default' : False,
            'dest'    : 'nolocal',
            'help'    : 'force remote files search (reverse of --local)'
         }
      },
      'vers' : {
         'args'   : ('--vers',),
         'kwargs' : {
            'type'    : 'string',
            'default' : run['default_vers'],
            'action'  : 'store',
            'dest'    : 'aster_vers',
            'metavar' : 'VERS',
            'help'    : 'Code_Aster version to used'
         }
      },
      'version_dev' : {
         'args'   : ('--version_dev',),
         'kwargs' : {
            'type'    : 'string',
            'default' : run['version_dev'],
            'action'  : 'store',
            'dest'    : 'version_dev',
            'metavar' : 'VERS',
            'help'    : 'development version of Code_Aster to update'
         }
      },
      'all' : {
         'args'   : ('-a','--all'),
         'kwargs' : {
            'default' : False,
            'action'  : 'store_true',
            'dest'    : 'all_test',
            'help'    : 'get all the files of the test'
         }
      },
      'test' : {
         'args'   : ('--test',),
         'kwargs' : {
            'default' : False,
            'action'  : 'store_true',
            'dest'    : 'test',
            'help'    : SUPPRESS_HELP
         }
      }
   }
   title='Options for maintenance operations'
   run.SetActions(
         actions_descr=acts_descr,
         actions_order=['getop','getversion','show','get','make','update', \
               'auto_update'],
         group_options=True,group_title=title,actions_group_title=False,
         options_descr=opts_descr
   )
   # additional check if remote command can occurs
   for par in 'devel_server_user','devel_server_ip':
      if run.config[par]=='' and not run['default_local']:
         run.Mess(_('remote connection may fail :\n%s not defined in %s') \
               % (par,run.user_rc),_('<A>_ALARM'))

#-------------------------------------------------------------------------------
def GetOP(run,*list_capy):
   """Return the main subroutine of a command.
   """
   REPREF=os.path.join(run['aster_root'],run['aster_vers'])
   if len(list_capy)<1:
      run.parser.error(
            _("'--%s' requires one or more arguments") % run.current_action)
   ficconf=os.path.join(REPREF,'config.txt')
   conf=ASTER_CONFIG(ficconf,run)
   for capy in list_capy:
      capy=string.replace(capy,'.capy','')
      nf=os.path.join(REPREF,conf['SRCCAPY'][0],'commande',capy+'.capy')
      if not os.path.exists(nf):
         run.Mess(_('file not found : %s') % nf,_('<A>_ALARM'))
         run.Mess(_('%s search capy files on local machine') \
               % run.current_action)
         break
      f=open(nf)
      nop=re.search('op *= *([-_0-9a-zA-Z]+)',f.read())
      if nop:
         name=nop.group(1)
         if re.search('[0-9]+',name):
            num='%04d' % int(name)
            name='op'+num+'.f'
         else:
            name=name+'.py'
         Show(run,name)
      else:
         run.Mess(_('op statement not found in %s') % capy,_('<A>_ALARM'))

#-------------------------------------------------------------------------------
def Show(run,*args):
   """Show a source file : fortran, c, python, capy, cata, histor or test.
   """
   kwargs={ 'get' : False }
   Get(run,*args,**kwargs)

#-------------------------------------------------------------------------------
def Get(run,*args,**kwargs):
   """Copy a source file (as Show) in current directory
   """
   if not kwargs.has_key('get'):
      get=True
   else:
      get=kwargs['get']

   if len(args)<1:
      run.parser.error(
            _("'--%s' requires one or more arguments") % run.current_action)

   if not run['local']:
      user=run['devel_server_user']
      mach=run['devel_server_ip']
      indv=run['aster_vers'][3:4]
      REPREF=os.path.join('/aster','v'+indv,run['aster_vers'])
      # rpertoires (en distant on suppose une organisation standard)
      RC   =os.path.join(REPREF,'bibc')
      RFORT=os.path.join(REPREF,'bibfor')
      RPY  =os.path.join(REPREF,'bibpyt')
      RHIST=os.path.join(REPREF,'histor')
      RCAPY=os.path.join(REPREF,'catapy')
      RCATA=os.path.join(REPREF,'catalo')
      RTEST=os.path.join(REPREF,'astest')
   else:
      user=os.environ['LOGNAME']
      mach=os.uname()[1]
      REPREF=os.path.join(run['aster_root'],run['aster_vers'])
      # rpertoires
      ficconf=os.path.join(REPREF,'config.txt')
      conf=ASTER_CONFIG(ficconf,run)
      RC   =os.path.join(REPREF,conf['SRCC'][0])
      RFORT=os.path.join(REPREF,conf['SRCFOR'][0])
      RPY  =os.path.join(REPREF,conf['SRCPY'][0])
      RHIST=os.path.join(REPREF,conf['SRCHIST'][0])
      RCAPY=os.path.join(REPREF,conf['SRCCAPY'][0])
      RCATA=os.path.join(REPREF,conf['SRCCATA'][0])
      RTEST=os.path.join(REPREF,conf['SRCTEST'][0])

   # ----- rpertoire temporaire
   if not get:
      rdest=run['cache_dir']
   else:
      rdest=os.getcwd()
   machdest=os.uname()[1]

   fmt = '-- %-44s [%+30s]'

   nberr=0
   for obj in args:
      # l'extension fournit le type sauf pour les histor
      ext=string.split(obj,'.')[-1]
      baseobj=string.replace(obj,'.'+ext,'',1)
      if re.search('[0-9]+\.[0-9]+\.[0-9]+',obj):
         ext='hist'
      rep=''
      srep=0
      ct=0
      if ext=='c':
         rep=RC
         srep=1
      elif ext=='f':
         rep=RFORT
         srep=1
      elif ext=='py':
         rep=RPY
         srep=1
      elif ext=='hist':
         rep=RHIST
      elif ext=='capy':
         rep=RCAPY
         srep=1
      elif ext=='cata':
         rep=RCATA
         srep=1
      elif ext=='comm':
         rep=RTEST
         ct=1
      
      if rep=='':
         print fmt % (obj,_('type unsupported'))
         nberr=nberr+1
      else:
         print fmt % (obj,rep)
         jret=0
         # le fichier existe ou on ne peut pas le vrifier
         if mach<>machdest or srep==1 or \
            (mach==machdest and os.path.isfile(os.path.join(rep,obj))):
         # 1. rcuprer le(s) fichier(s)
            if run['all_test'] and ct==1:
            # cas-test : all
               # est-il dans le cache ?
               if not run['force'] and \
                     os.path.isdir(os.path.join(rdest,baseobj)):
                  print '  |  '+_('from cache')
               else:
                  if not os.path.isdir(os.path.join(rdest,baseobj)):
                     os.mkdir(os.path.join(rdest,baseobj))
                  src=os.path.join(rep,baseobj)+'.*'
                  if mach==machdest:
                     print '  |  '+_('copy all files of the test')
                  else:
                     print '  |  '+_('remote copy of the test files')
                     src=user+'@'+mach+':'+src
                  jret=run.Copy(os.path.join(rdest,baseobj),src)
            else:
            # fichier individuel
               # est-il dans le cache ?
               if not run['force'] and \
                     os.path.isfile(os.path.join(rdest,obj)):
                  print '  |  '+_('from cache')
               else:
                  if srep==1:
                     src=os.path.join(rep,'*',obj)
                  else:
                     src=os.path.join(rep,obj)
                  if mach==machdest:
                     print '  |  '+_('copy')
                  else:
                     print '  |  '+_('remote copy')
                     src=user+'@'+mach+':'+src
                  jret=run.Copy(rdest,src)

         # 2. test / edition
            if jret<>0:
               print '  |  '+_('error occurs during copying')
               nberr=nberr+1
            else:
               # si show
               if not get:
                  if run['all_test'] and ct==1:
                  # cas-test : all
                     print '  |  '+_('edit files')
                     cmd=run['editor']+' '+ \
                           os.path.join(rdest,baseobj,'*')+' &'
                  else:
                  # fichier individuel
                     print '  |  '+_('edit file')
                     cmd=run['editor']+' '+os.path.join(rdest,obj)+' &'
                  jret,output=run.Shell(cmd,bg=True)
         else:
         # fichier inexistant
            print ' |   '+_('file not found')
            nberr=nberr+1

   if nberr>0:
      if nberr>1: s='s'
      else: s=''
      run.Mess(_('%d error(s) detected') % nberr,_('<A>_ALARM'))

#-------------------------------------------------------------------------------
def GetVersion(run,*args,**kwargs):
   """Return release number of current development version
   """
   if len(args)>0:
      run.parser.error(_("'--%s' requires no argument") % run.current_action)
   # ----- default keywords
   silent=False
   REPREF=os.path.join(run['aster_root'],run['aster_vers'])
   if kwargs.has_key('silent'):
      silent=kwargs['silent']
   if kwargs.has_key('REPREF'):
      REPREF=kwargs['REPREF']
   ficconf=os.path.join(REPREF,'config.txt')
   # if file don't exist, suppose default value
   if os.path.exists(ficconf):
      conf=ASTER_CONFIG(ficconf,run)
      bibpyt=conf['SRCPY'][0]
   else:
      bibpyt='bibpyt'
   # ----- properties.py
   f=os.path.join(REPREF,bibpyt,'Accas','properties.py')
   if not os.path.isfile(f):
      run.Mess(_('file not found : %s') % f,_('<F>_FILE_NOT_FOUND'))
      return
   mydict={}
   execfile(f,mydict)
   result=mydict['version'].split('.')
   result.append(mydict['date'])
   iret=0
   if len(result) <> 4:
      iret=4
   elif not silent:
      print _('Version %s - %s') % (string.join(result[:3],'.'),result[3])
   return iret,result

#-------------------------------------------------------------------------------
def MakeAster(run,*args):
   """Build Code_Aster from sources
   """
   REPREF=os.path.join(run['aster_root'],run['version_dev'])
   fconf=os.path.join(REPREF,'config.txt')
   conf=ASTER_CONFIG(fconf,run)
   build=ASTER_BUILD(run,conf)

   # check arguments
   if len(args) > 0:
      run.parser.error(_("'--%s' requires no argument") % run.current_action)

   run.print_timer=True

   # 0. ----- working directory during build
   reptrav = os.path.join(run['tmp_user'],'build.'+run['num_job'])
   run.to_delete.append(reptrav)
   run.MkDir(reptrav)

   # 1. ----- Go !
   _build_aster(run,False,REPREF,reptrav)
   run.Delete(reptrav)

#-------------------------------------------------------------------------------
def _build_aster(run,iupdate,REPREF,reptrav,lardv=[]):
   """Build or update Aster depending of iupdate value.
      iupdate : False for 'make', True for 'update'
      lardv   : list of object files to remove from lib_aster
   """
   fconf=os.path.join(REPREF,'config.txt')
   conf=ASTER_CONFIG(fconf,run)
   build=ASTER_BUILD(run,conf)
   DbgPara={
      'debug'     : { 'exe'  : conf['BIN_DBG'][0],
                      'suff' : conf['BINOBJ_DBG'][0]},
      'nodebug'   : { 'exe'  : conf['BIN_NODBG'][0],
                      'suff' : conf['BINOBJ_NODBG'][0]},
   }

   # 1a. ----- perform update twice : debug and nodebug modes
   # first, remove old .o files
   for mode in DbgPara.keys():
      replib = os.path.join(REPREF,'lib_'+DbgPara[mode]['suff'])
      run.Mess(_('Start build in %s mode') % mode,'TITLE')      

      # 1.1. remove object files from lib_aster
      if iupdate:
         run.timer.Start('Deletion of old files')
         # 'ar -dv' not provided in config.txt
         cmd=[ conf['LIB'][0].split()[0], '-dv',
               os.path.join(replib,'lib_aster.lib') ]
         cmd.extend(lardv)
         run.VerbStart(_('remove object files from %s') % \
               'lib_'+DbgPara[mode]['suff'],verbose=True)
         if lardv:
            if run['verbose']: print
            kret, out = run.Shell(string.join(cmd))
            # ----- avoid to duplicate output
            if not run['verbose']:
               run.VerbEnd(kret,output=out,verbose=True)
            if kret<>0:
               run.Mess(_('error during deleting objects from archive'),
                     _('<E>_ARCHIVE_ERROR'))
         else:
            run.VerbIgnore(verbose=True)
         run.timer.Stop('Deletion of old files')
         run.CheckOK()
   
   # 1b. ----- perform update twice : debug and nodebug modes
   for mode in DbgPara.keys():
      replib = os.path.join(REPREF,'lib_'+DbgPara[mode]['suff'])
      run.Mess(_('Start build in %s mode') % mode,'TITLE')      

      # 1.2. compile updated files
      run.timer.Start('Compilation in %s mode' % mode)
      kret = build.CompilAster(REPREF,dbg=mode,ignore_ferm=iupdate)
      run.timer.Stop('Compilation in %s mode' % mode)
      run.CheckOK()

      # 1.3. archive '.o'
      # 1.3.1. obj or dbg
      run.timer.Start('Add object files to library')
      kret = build.Archive(repobj=os.path.join(REPREF,DbgPara[mode]['suff']),
            lib=os.path.join(replib,'lib_aster.lib'))
      run.CheckOK()

      # 1.3.2. obj_f or dbg_f
      if not iupdate:
         kret = build.Archive(
               repobj=os.path.join(REPREF,DbgPara[mode]['suff']+'_f'),
               lib=os.path.join(replib,'ferm.lib'))
         run.CheckOK()
      run.timer.Stop('Add object files to library')

      # 1.3.3. copy 'main.o' near lib_aster
      pyto = os.path.join(replib,'python.o')
      objp = os.path.join(REPREF,DbgPara[mode]['suff'],'python.o')
      if not os.path.exists(pyto) or \
            os.stat(objp).st_ctime > os.stat(pyto).st_ctime:
         run.Copy(pyto, objp)

      # 1.4. update executable
      run.timer.Start('Build executables')
      lobj = [pyto,
              os.path.join(replib,'lib_aster.lib'),
              os.path.join(replib,'ferm.lib')]
      kret = build.Link(os.path.join(REPREF,DbgPara[mode]['exe']), lobj)
      run.timer.Stop('Build executables')
      run.CheckOK()

   # 2. ----- compile commands
   run.timer.Start('Compilation of commands catalog')
   kret = build.CompilCapy(REPREF,reptrav)
   run.timer.Stop('Compilation of commands catalog')
   run.CheckOK()

   # 2. ----- compile elements
   kargs={
      'exe'    : os.path.join(REPREF,DbgPara['nodebug']['exe']),
      'cmde'   : os.path.join(REPREF,conf['BINCMDE'][0]),
   }
   run.timer.Start('Make pickled of elements')
   kret = build.MakePickled(REPREF,reptrav)
   run.timer.Stop('Make pickled of elements')
   run.CheckOK()

   run.timer.Start('Compilation of elements')
   kret = build.CompilEle(REPREF,reptrav,**kargs)
   run.timer.Stop('Compilation of elements')
   run.CheckOK()

   # 9. ----- end
   run.Mess(_('Code_Aster has been successfully built'),'OK')

#-------------------------------------------------------------------------------
def Update(run,*args):
   """Extract "aster-maj" archives and make the update
   """
   REPREF=os.path.join(run['aster_root'],run['version_dev'])   
   fconf=os.path.join(REPREF,'config.txt')
   conf=ASTER_CONFIG(fconf,run)
   build=ASTER_BUILD(run,conf)

   # check arguments
   if len(args)<1:
      run.parser.error(
            _("'--%s' requires one or more arguments") % run.current_action)

   run.print_timer=True
   larch=[]
   for arch in args:
      larch.append(os.path.abspath(arch))

   # 0. ----- working directory during update(s)
   prefix='maj'
   repmaj=os.path.join(run['tmp_user'],prefix)
   run.to_delete.append(repmaj)
   run.MkDir(repmaj)
   reptrav = os.path.join(run['tmp_user'],'update.'+run['num_job'])
   run.to_delete.append(reptrav)
   run.MkDir(reptrav)

   run.timer.Start('Extraction of archives')
   # 1. ----- for each archive
   os.chdir(run['tmp_user'])
   funig=os.path.join(repmaj,'unigest')
   lupd=[]
   lunig=[]
   iret=0
   for arch in larch:
      arch=os.path.abspath(arch)
      if tarfile.is_tarfile(arch):
         is_unigest=False
         linfo=[]

         # 1.1. ----- extraction
         run.VerbStart(_('extract archive %s') % arch,verbose=True)
         jret=0
         try:
            tar=tarfile.open(arch,'r')
            tar.errorlevel=2
            for ti in tar:
               tar.extract(ti)
               # if name is in a previous unigest don't delete it
               name=re.sub('^'+prefix+'/','',ti.name)
               if os.path.basename(name)<>'' and name<>'unigest':
                  lupd.append(name)
               if name in lunig:
                  linfo.append(name)
                  lunig.remove(name)
         except tarfile.ExtractError:
            jret=4
         run.VerbEnd(jret,verbose=True)
         tar.close()
         if jret<>0:
            run.Mess(_('error during extracting archive %s') % arch,
                  _('<F>_TAR_ERROR'))

         # 1.2. ----- give info about files not to delete
         if linfo:
            run.Mess(_('These files should have previously been deleted ' \
                  'and now they are updated :'))
            print ', '.join(linfo)

         # 1.3. ----- get directives from unigest
         linfo=[]
         if os.path.exists(funig):
            for k, val in build.GetUnigest(funig).items():
               if k<>'filename':
                  lunig.extend(val)
            # delete twins
            if len(lunig)>0:
               ilast=lunig[0]
               for it in lunig[1:]:
                  if it==ilast:
                     lunig.remove(it)
            os.remove(funig)
            # delete files from maj which have been modified since last update
            for f in lunig:
               if f in lupd:
                  lupd.remove(f)
               f=os.path.join(repmaj,f)
               if os.path.exists(f):
                  linfo.append(f)

         # 1.4. ----- give info about files to delete before update
         if linfo:
            run.Mess(_('These files have been modified by a previous update ' \
                  'but now they are deleted :'))
            for f in linfo:
               run.Delete(f, verbose=True)

      else:
         run.Mess(_('invalid tar (compressed) archive or ' \
               'file not found : %s') % arch,_('<F>_FILE_NOT_FOUND'))
   run.timer.Stop('Extraction of archives')

   # 2. ----- start update
   # current version
   i,vvv=GetVersion(run,silent=True,REPREF=REPREF)
   if i<>0:
      run.Mess(_('error occurs during getting release number'),_('<F>_ERROR'))
   old_vers=string.join(vvv[:3],'.')
   # target version
   i,vnext=GetVersion(run,silent=True,REPREF=repmaj)
   if i<>0:
      run.Mess(_('error occurs during getting release number'),_('<F>_ERROR'))
   new_vers=string.join(vnext[:3],'.')
   run.Mess(_('Update version %s to %s') % (old_vers,new_vers),'TITLE')
   
   # 3. ----- copy files
   run.timer.Start('Copy of files')
   run.Mess(_('Copy updated files'),'TITLE')
   os.chdir(repmaj)
   for rep,suff in { conf['SRCFOR'][0]  : '*.f',  conf['SRCC'][0]    :'*.c',
                     conf['SRCPY'][0]   : '*.py', conf['SRCCAPY'][0] :'*.capy',
                     conf['SRCCATA'][0] : '*.cata' }.items():
      if os.path.isdir(rep):
         for entry in glob.glob(os.path.join(rep,'*/')):
            src  = os.path.join(entry,suff)
            dest = os.path.join(REPREF,entry)
            run.MkDir(dest, verbose=False)
            run.Copy(dest, src)
   run.timer.Stop('Copy of files')

   # 4. ----- delete source files
   run.timer.Start('Deletion of old files')
   run.Mess(_('Apply unigest directives'),'TITLE')
   lardv=[]
   for f in lunig:
      run.Delete(os.path.join(REPREF,f),verbose=True)
      if re.search('^'+conf['SRCC'][0]+'/|^'+conf['SRCFOR'][0]+'/',f):
         lardv.append(os.path.basename(re.sub('\.[cf]+$','.o',f)))
   run.timer.Stop('Deletion of old files')

   # 5. ----- update executable, libs and catalogs
   _build_aster(run,True,REPREF,reptrav,lardv=lardv)
   run.Delete(reptrav)

#-------------------------------------------------------------------------------
def AutoUpdate(run,*args):
   """Try to download available updates and apply them.
   """
   if len(args)>0:
      run.parser.error(_("'--%s' requires no argument") % run.current_action)

   run.print_timer=True
   if not run['local']:
      if run['update_protocol']=='HTTP':
         REPSRC=run['http_server_user']+'@'+run['http_server_ip']+ \
               ':'+run['http_rep_maj']
      else:
         REPSRC=run['devel_server_user']+'@'+run['devel_server_ip']+ \
               ':'+run['devel_rep_maj']
   else:
      REPSRC=run['local_rep_maj']

   iret=0
   # version stabilise courante (ex : 7.2)
   run.timer.Start('Download update archives')
   i,v=GetVersion(run,silent=True, \
         REPREF=os.path.join(run['aster_root'],run['version_dev']))
   if i<>0:
      run.Mess(_('error occurs during getting release number'),_('<F>_ERROR'))
   sta=v[0]+'.'+v[1]
   vlast=sta + '.' + v[2]
   run.Mess(_('Searching updates of %s following release %s...')\
         % (sta,vlast))

   reptrav = os.path.join(run['tmp_user'],'auto_update.'+run['num_job'])
   run.to_delete.append(reptrav)
   run.MkDir(reptrav)
   os.chdir(reptrav)
   lmaj=[]
   # ----- search for sub-releases of current version
   for i in [0,1]:
      for j in range((1-i)*int(v[2])+1,50):
         vch=v[0]+'.'+str(int(v[1])+i)+'.'+str(j)
         # nom du fichier archive
         src=os.path.join(REPSRC,'aster-maj-src-'+vch+'-1.tar.gz')
         dest=os.path.join(reptrav,os.path.basename(src))
         # use niverr='INFO' to ignore failure
         jret=run.Copy(dest, src, verbose=True, niverr='INFO', \
               protocol=run['update_protocol'])
         if jret<>0:
            if i==1:
               run.Mess(_('Last release found : %s') % vlast + '\n')
            else:
               nextsta=v[0]+'.'+str(int(v[1])+1)
               run.Mess(_('Testing if STA %s was released...') % nextsta)
            break
         else:
            vlast=vch
            lmaj.append(dest)
   run.timer.Stop('Download update archives')

   # ----- start a multiple update
   if lmaj:
      iret=Update(run,*lmaj)
   else:
      run.Mess(_('no update found'),'OK')
   run.Delete(reptrav)

