# -*- coding: iso-8859-1 -*-

# ==============================================================================
# COPYRIGHT (C) 1991 - 2003  EDF R&D                  WWW.CODE-ASTER.ORG
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
# IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
# THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
# (AT YOUR OPTION) ANY LATER VERSION.
#
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
# WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.
#
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
# ==============================================================================

# $Id: as_timer.py,v 1.3 2004/10/15 13:10:04 mcourtoi Exp $
# $Name: v1_2_1 $

# ----- differ messages translation
def _(mesg): return mesg

import time

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
class ASTER_TIMER:
   """This class provides methods to easily measure time spent during
   different steps.
   Methods :
      Start : start a timer in mode 'INIT' ([re]start from 0) or 'CONT'
         (restart from last value).
      Stop  : stop a timer
   Attributes :
      timers : dict {
         timer_name : {
            'state' : state,
            't0' : start_time,
            'dt' : spent_time,
            'num': timer number (to print timers in order of creation),
         },
         ...
      }
         state is one of 'start', 'stop'
   """
   MaxNumTimer = 9999999
   TotalKey = _('Total time')
   Elapsed = _('elapsed time')

#-------------------------------------------------------------------------------
   def __init__(self,add_total=True,format='s'):
      """Constructor
      """
      # ----- initialisation
      self.timers={}
      self.total=add_total
      if self.total:
         self.Start(self.TotalKey,num=self.MaxNumTimer)
      self.format=format
      if not format in ('s','hms'):
         self.format='s'

#-------------------------------------------------------------------------------
   def Start(self,timer,mode='CONT',num=None):
      """Start a new timer or restart one
      """
      isnew=not timer in self.timers.keys()
      if not num:
         num = len(self.timers)
      if isnew:
         self.timers[timer]={
            'state' : 'start',
            't0' : time.time(),
            'dt' : 0.,
            'num': num,
         }
      elif mode=='INIT':
         self.timers[timer]={
            'state' : 'start',
            't0' : time.time(),
            'dt' : 0.,
            'num': self.timers[timer]['num'],
         }
      elif mode=='CONT' and self.timers[timer]['state']=='stop':
         self.timers[timer]['state'] = 'start'
         self.timers[timer]['t0'] = time.time()

#-------------------------------------------------------------------------------
   def Stop(self,timer):
      """Stop in timer
      """
      if not timer in self.timers.keys():
         self.timers[timer] = {
            'state' : 'stop',
            't0' : 0.,
            'dt' : 0.,
            'num': len(self.timers),
         }
      elif self.timers[timer]['state'] == 'start':
         self.timers[timer]['state'] = 'stop'
         self.timers[timer]['dt'] = self.timers[timer]['dt'] + \
               time.time() - self.timers[timer]['t0']
         self.timers[timer]['t0'] = 0.

#-------------------------------------------------------------------------------
   def StopAll(self):
      """Stop all timers
      """
      lk=self.timers.keys()
      if self.total:
         lk.remove(self.TotalKey)
      for timer in lk:
         self.Stop(timer)

#-------------------------------------------------------------------------------
   def __repr__(self):
      """Pretty print content of the timer.
      NB : call automatically StopAll
      """
      self.StopAll()
      self.Stop(self.TotalKey)
      # formats
      label_size=min(80-21,max([len(k) for k in self.timers.keys()]))
      sepa = ' '+'-'*(2+label_size+3+14+2)
      fmthms  = '   %-'+str(label_size)+'s   %02dh %02dm %5.2fs  '
      fmts    = '   %-'+str(label_size)+'s   %12.2f s  '
      out=[]
      # get timers list and sort by 'num'
      lnum = [[val['num'],timer] for timer,val in self.timers.items()]
      lnum.sort()
      if lnum:
         out.append(sepa)
      for num, timer in lnum:
         dt  = self.timers[timer]['dt']
         h=int(dt/3600)
         m=int(dt % 3600)/60
         s=(dt % 3600) % 60
         if self.total and num==self.MaxNumTimer and len(lnum)>1:
            out.append(sepa)
         if self.format=='s':
            out.append(fmts % (timer,dt))
         elif self.format=='hms':
            out.append(fmthms % (timer,h,m,s))
         if self.total and num==self.MaxNumTimer:
            out.append('   (%s)' % self.Elapsed)
      if lnum:
         out.append(sepa)
      return '\n'.join(out)

#-------------------------------------------------------------------------------
if __name__ == '__main__':
   timer=ASTER_TIMER()
   timer.Start('Build Code_Aster')
   timer.Start('Compilation')
   time.sleep(1.4)
   timer.Stop('Compilation')
   timer.Stop('Catalogs')
   time.sleep(2.1)
   timer.Start('Compilation')
   time.sleep(1.1)
   timer.Stop('Compilation')
   print timer
   print 'Excepting :'
   print 'build : 4.6'
   print 'compilation : 2.5'
   print 'catalogs : 0.'
